const _ = require('lodash');
const pluralize = require('pluralize');
const url = require('url');

const ActionTemplate = (wordForms, properties, urlBase = '') => `
	import validate from '../validations/${wordForms.singular.upperFirst}';

	export const create${wordForms.singular.upperFirst} = ${wordForms.singular.camelCase} => {

	    const errors = validate(${wordForms.singular.camelCase});

	    if (errors) {
	        return {
	            type: 'CREATE_${wordForms.singular.snakeCase.toUpperCase()}_FAILED',
	            errors: errors
	        };
	    } else {
	        return {
	            type: 'CREATE_${wordForms.singular.snakeCase.toUpperCase()}',
	            url: '${urlBase}/${wordForms.plural.camelCase}',
	            method: 'post',
	            body: ${wordForms.singular.camelCase}
	        };
	    }
	};

	export const get${wordForms.singular.upperFirst} = id => {
	    return {
	        type: 'GET_${wordForms.singular.snakeCase.toUpperCase()}',
	        url: \`${urlBase}/${wordForms.plural.camelCase}/\${id}\`,
	        method: 'get'
	    };
	};

	export const update${wordForms.singular.upperFirst} = (id, ${wordForms.singular.camelCase}) => {

	    const errors = validate(${wordForms.singular.camelCase});

	    if (errors) {
	        return {
	            type: 'UPDATE_${wordForms.singular.snakeCase.toUpperCase()}_FAILED',
	            errors: errors
	        };
	    } else {
	        return {
	            type: 'UPDATE_${wordForms.singular.snakeCase.toUpperCase()}',
	            url: \`${urlBase}/${wordForms.plural.camelCase}/\${id}\`,
	            method: 'put',
	            body: ${wordForms.singular.camelCase}
	        };
	    }
	};

	export const delete${wordForms.singular.upperFirst} = id => {
	    return {
	        type: 'DELETE_${wordForms.singular.snakeCase.toUpperCase()}',
	        url: \`${urlBase}/${wordForms.plural.camelCase}/\${id}\`,
	        method: 'delete'
	    };
	};

	export const set${wordForms.singular.upperFirst} = ${wordForms.singular.camelCase} => {
	    return {
	        type: 'SET_${wordForms.singular.snakeCase.toUpperCase()}',
	        data: ${wordForms.singular.camelCase}
	    };
	};
`;

module.exports = ActionTemplate;
