const ApiTemplate = () => `
	import axios from 'axios';

	const defaultHeaders = {
		Accept: 'application/json',
		'Content-Type': 'application/json'
	};

	export default store => dispatch => action => {
		const { url, type, method = 'get', body = {}, params = {}, headers = {} } = action;

		if (!url) {
			return dispatch(action);
		}

		dispatch({
			...action,
			type: \`\$\{type\}_START\`
		});

		return axios({
			method,
			url,
			data: body,
			params,
			headers: {
				...defaultHeaders,
				...headers
			}
		})
			.then(({ data }) =>
				dispatch({
					type: \`\$\{type\}_SUCCESS\`,
					data,
					url,
					params
				})
			)
			.catch(({ status, stack, data, errors }) =>
				dispatch({
					type: \`\$\{type\}_FAIL\`,
					data,
					errors,
					status,
					stack
				})
			);
	};
`;

module.exports = ApiTemplate;
