const _ = require('lodash');
const pluralize = require('pluralize');

const ValidationTemplate = (wordForms, properties) => {
	const propertyTitles = Object.keys(_.omit(properties, 'id')); // omit id

	const tplHeader = `		export default ({${propertyTitles.join(', ')}}) => {
			let errors = null;
`;

	let tplBody = ``;

	propertyTitles.map(propertyTitle => {
		let capitalized = _.upperFirst(propertyTitle);

		tplBody += `\t/* Validations for ${propertyTitle} */\n\n`;

		// 6.17. required
		if (properties[propertyTitle].required) {
			tplBody += `
			if (!${propertyTitle}) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} is required.';
			}\n`;
		}

		// 6.1. multipleOf
		if (properties[propertyTitle].multipleOf) {
			// TODO: add division by zero checking
			tplBody += `
			if (!(!isNaN(value) && (function(x) { return (x | 0) === x; })(parseFloat(${propertyTitle} / ${properties[propertyTitle].multipleOf})))) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} should be multiple of ${properties[propertyTitle].multipleOf}.';
			}\n`;
		}

		// 6.2. maximum
		if (properties[propertyTitle].maximum) {
			tplBody += `
			if (${propertyTitle} > ${properties[propertyTitle].maximum}) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} should be less than or equal ${properties[propertyTitle].maximum}.';
			}\n`;
		}

		// 6.3. exclusiveMaximum

		// 6.4. minimum
		// TODO: Add in range if both maximum and minimum present
		if (properties[propertyTitle].minimum) {
			tplBody += `
			if (${propertyTitle} < ${properties[propertyTitle].minimum}) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} should be greather than or equal ${properties[propertyTitle].minimum}.';
			}\n`;
		}

		// 6.5. exclusiveMinimum

		// 6.6. maxLength
		if (properties[propertyTitle].maxLength) {
			tplBody += `
			if (${propertyTitle}.length > ${properties[propertyTitle].maxLength}) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} length is too long. It should be less than or equal ${properties[propertyTitle].maxLength}.';
			}\n`;
		}

		// 6.7. minLength
		if (properties[propertyTitle].minLength) {
			tplBody += `
			if (${propertyTitle}.length < ${properties[propertyTitle].minLength}) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} length is too short. It should be more than or equal ${properties[propertyTitle].minLength}.';
			}\n`;
		}

		// 6.8. pattern
		if (properties[propertyTitle].pattern) {
			tplBody += `
			const ${propertyTitle}Pattern = new RegExp(${properties[propertyTitle].pattern});

			if (!${propertyTitle}Pattern.test(${propertyTitle})) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} is incorrect.';
			}\n`;
		}

		// 6.9. items
		// 6.10. additionalItems

		// 6.11. maxItems
		if (properties[propertyTitle].maxItems) {
			tplBody += `
			if (${propertyTitle}.length > ${properties[propertyTitle].maxItems}) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} should contain less than or equal ${properties[propertyTitle].maxItems} items.';
			}\n`;
		}

		// 6.12. minItems
		if (properties[propertyTitle].minItems) {
			tplBody += `
			if (${propertyTitle}.length < ${properties[propertyTitle].minItems}) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} should contain more than or equal ${properties[propertyTitle].minItems} items.';
			}\n`;
		}

		// 6.13. uniqueItems
		// 6.14. contains
		// 6.15. maxProperties
		// 6.16. minProperties

		// 6.18. properties
		// 6.19. patternProperties
		// 6.20. additionalProperties
		// 6.21. dependencies

		// 6.22. propertyNames

		// 6.23. enum
		if (properties[propertyTitle].enum) {
			tplBody += `
			if (!~["${properties[propertyTitle].enum.join('" ,"')}"].indexOf(${propertyTitle})) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} should be one of ${properties[propertyTitle].enum}.';
			}\n`;
		}

		// 6.24. const
		if (properties[propertyTitle].const) {
			tplBody += `
			if (${propertyTitle} !== ${properties[propertyTitle].const}) {
				errors = errors || {};
				errors['${propertyTitle}'] = '${capitalized} should be ${properties[propertyTitle].const}.';
			}\n`;
		}

		// 6.25. type
		// 6.26. allOf
		// 6.27. anyOf
		// 6.28. oneOf
		// 6.29. not
	});

	const tplFooter = `
			return errors;
		};
	`;

	const tpl = tplHeader + tplBody + tplFooter;

	return tpl.replace(/^\t{2}/gm, '');
};

module.exports = ValidationTemplate;
